#!/bin/sh
# jw@lxsbbs.eu/teamaster@teanet.org, script for free use
# usage ./cert_srv.sh teadrinker.tld

if [ ! -e ./ownssl.conf.sh ]; then
    echo "cannot found ownssl.conf.sh"
    echo "please change to ownssl-dir"
    echo "cd /wherever/ownssl or where it is"
    exit
fi

. ./ownssl.conf.sh

if [ "$letsselfsigned_copy" = "true" ]; then
    if [ ! -d "$letsselfsigned_dir/live" ]; then
	echo "letsselfsigned_dir $letsselfsigned_dir/live not found";
	echo "erstellen? y=yes, a=abort"
	read antwort
	if [ "$antwort" = "y" ]; then
	    mkdir -p "$letsselfsigned_dir"
	    mkdir -p "$letsselfsigned_dir/live"
	elif [ "$antwort" = "a" ]; then
	    echo "abort..."; exit 1
	else
	    echo "letsselfsigned_dir $letsselfsigned_dir/live not found";
	    echo "create or set letsselfsigned_copy=\"true\""
	    echo "abort..."; exit 1
	fi
    fi
fi
if [ "${1}" = "help" ]; then usage; exit; fi
noclient_flag="false";
s_domain_param="";
if [ $# -gt 1 ]; then
    if [ "$1" = "noclient" ]; then
	noclient_flag="true"; s_domain_param="$2"
    elif [ "$2" = "noclient" ]; then
	noclient_flag="true"; s_domain_param="$1"
    fi
else
    s_domain_param="$1"
fi

if ! is_domainname "$1"; then
    echo "param: $s_domain_param"
    echo "domain required"
    echo "enter domain, or key to exit"
    read s_domain
else
    s_domain="${s_domain_param}"
fi

if [ -z "$s_domain" ]; then echo "$domain is missing...abort..."; exit; fi
genconf_wildcard "$s_domain"

if [ ! -e "$ssldir/${s_caname}.key" ]; then
    echo "ca $ssldir/${s_caname}.key not found."; exit
fi
if [ ! -e "$ssldir/${s_caname}.crt" ]; then
    echo "ca $ssldir/${s_caname}.crt not found."; exit
fi
if [ ! -e "$ssldir/cnf.${s_domain}.key" ] || [ ! -e "$ssldir/cnf.${s_domain}.srv" ]; then
    echo "ca $ssldir/cnf.${s_domain}.key/srv not found."
    echo "evt. gen.srv.cnf.sh aufrufen...abort..."
    exit
fi

echo "ca ${s_caname}.key, ${s_caname}.crt"
echo "cnf.${s_domain}.key, cnf.${s_domain}.srv"
echo "server certificate create?  y = yes, doit"
read antwort
antwort=`echo $antwort | tr [:upper:] [:lower:]`
if [ "$antwort" != "y" ]; then
    echo "not confirmed, cancel"; exit 1
fi

# srv_key
rm -f ./${s_domain}.csr
rm -f ./privkey.pem
#echo "openssl req -new -sha256 -nodes -out ${s_domain}.csr -newkey rsa:4096 -config cnf.${s_domain}.srv"
openssl req -new -sha256 -nodes -out ${s_domain}.csr -newkey rsa:2048 -config cnf.${s_domain}.key
# -> ${s_domain}.csr privkey.pem

# srv_crt
rm -f fullcahin.pem
#openssl ca -config openssl-ca_sign.cnf -policy signing_policy -extensions signing_req -out cert.pem -infiles ${s_domain}.csr
openssl x509 -req -in ${s_domain}.csr -CA ${s_caname}.crt -CAkey ${s_caname}.key -CAcreateserial -extfile cnf.${s_domain}.srv -out ${s_domain}.crt -days 3650 -sha256
# -> ${s_domain}.crt

if [ ! -e "$ssldir/${s_caname}.p12" ]; then
    #openssl pkcs12 -export -inkey privkey.pem -in ll.lxsbbs.eu.crt -out client.p12
    #openssl pkcs12 -export -inkey privkey.pem -in ${s_domain}.crt -out ${s_domain}.p12
    openssl pkcs12 -export -inkey privkey.pem -in ${s_domain}.crt -out ${s_caname}.p12
    client_key_flag="true"
else
    client_key_flag="false"
fi



echo ""
log_date=`date '+%y%m%d.%H%M%S'`

if [ "$client_key_flag" = "true" ]; then
    destcl="${ssldir}/client_cert_${log_date}.${s_caname}"
    mkdir -p "$destcl"
    #mv -f "${s_domain}.p12" "${dest}/${s_domain}.p12"
    cp -f "${s_caname}.p12" "${destcl}/${s_caname}.p12"
fi

dest="${ssldir}/srv_cert_${log_date}.${s_domain}"
destssl="${dest}/${s_domain}"
mkdir -p "$destssl"

cp -af "${s_domain}.crt" fullchain.pem
cat ${s_caname}.crt >> fullchain.pem

mv -f "${s_domain}.crt" "${destssl}/cert.pem"
cp -af "${s_caname}.crt" "${destssl}/chain.pem"
mv -f "fullchain.pem" "${destssl}/fullchain.pem"
mv -f "privkey.pem" "${destssl}/privkey.pem"
if [ ! -d "$letsselfsigned_dir/live" ]; then
    echo "letsselfsigned_dir $letsselfsigned_dir/live not found";
    echo "erstellen? y=yes, a=abort"
else
    if [ "$letsselfsigned_copy" = "true" ]; then
	cp -afrv "${destssl}/" "${letsselfsigned_dir}/live/"
    fi
fi
mv -f "cnf.${s_domain}.key" "${dest}/cnf.${s_domain}.key"
mv -f "cnf.${s_domain}.srv" "${dest}/cnf.${s_domain}.srv"
#cp -f "${s_domain}.csr" "${dest}/${s_domain}.csr"
rm -f "${s_domain}.csr"
echo "cert.pem das siegnierte server cert"
echo "chain.pem - CA-Cert"
echo "privkey.pem"
echo "fullchain.pem (cert.pem+chain.pem)"
echo "saved/gesichert nach $dest"
echo "namenschema wie bei letsencrypt"
echo "naming scheme like letsencrypt"

exit
